@@ -1,4 +1,5 @@
 from time import sleep, time
+from math import ceil
 
 from psutil import Process, cpu_count, cpu_times
 
@@ -11,6 +12,7 @@ CPU_IDLE_POLL_MIN_INTERVAL = 0.050
 
 __all__ = ["wait_for_idle_cpus"]
 
+
 class MovingAverage:
     def __init__(self, init=0, inertia=0.6):
         self.value = init
@@ -33,10 +35,13 @@ class WorkersRunning:
     def done(self):
         self.count -= 1
 
-'''Timed measurement on an incrementing counter
+
+"""Timed measurement on an incrementing counter
 
 Reads the counter, divides with time passed since last reading, and smooths the value using a moving average
-'''
+"""
+
+
 class InertialTimeDerivate:
     def __init__(self, func, initial, inertia):
         self.func = func
@@ -53,29 +58,40 @@ class InertialTimeDerivate:
         self.last_value = new_value
         return self.inertial_value.update(delta / elapsed)
 
-'''CPU-gated wait_for_slot implementation
+
+"""CPU-gated wait_for_slot implementation
 
 Creates a wait_for_slot-callable that will let through at least one concurrent job, and additional jobs as
 long as `count` cpu:s are idle. It is cgroup-aware, and will not allow current process and subprocesses to
 consume more CPU than configured cgroup-limit.
-'''
-def wait_for_idle_cpus(required, *, fixed_delay=0.2, inertia=0.7):
+"""
+
+
+def wait_for_idle_cpus(required, *, max_concurrent=None, fixed_delay=0.2, inertia=0.7):
     workers = WorkersRunning()
     cpu_poll_interval = MovingAverage(1, 0.7)
 
     process = Process()
+
     def cpu_used_in_process():
         times = process.cpu_times()
         return times.user + times.system + times.children_user + times.children_system
 
-    with open('/sys/fs/cgroup/cpu/cpu.cfs_quota_us', 'rt') as quota, open('/sys/fs/cgroup/cpu/cpu.cfs_period_us', 'rt') as period:
+    with open("/sys/fs/cgroup/cpu/cpu.cfs_quota_us", "rt") as quota, open(
+        "/sys/fs/cgroup/cpu/cpu.cfs_period_us", "rt"
+    ) as period:
         quota = int(quota.read())
         if quota > 0:
-            cpu_quota = (quota / int(period.read()))
+            cpu_quota = quota / int(period.read())
+            if not max_concurrent:
+                max_concurrent = ceil(cpu_quota / required)
+
             # Precalculate the quota with headroom for a new job
             cpu_quota -= required
         else:
             cpu_quota = None
+            if not max_concurrent:
+                max_concurrent = ceil(cpu_count() / required)
 
     def wait_for_slot(halt):
         if workers.count > 0:
@@ -87,10 +103,14 @@ def wait_for_idle_cpus(required, *, fixed_delay=0.2, inertia=0.7):
         if cpu_quota:
             cpu_used = InertialTimeDerivate(cpu_used_in_process, cpu_count(), inertia)
         update_interval = max(cpu_poll_interval.value, CPU_IDLE_POLL_MIN_INTERVAL)
-        while not halt():
+        while workers.count > 0 and not halt():
             sleep(update_interval)
 
-            if idle.update() >= required and (cpu_quota is None or cpu_used.update() <= cpu_quota):
+            if (
+                workers.count < max_concurrent
+                and idle.update() >= required
+                and (cpu_quota is None or cpu_used.update() <= cpu_quota)
+            ):
                 cpu_poll_interval.update((time() - start_time) / 20)
                 break
 
